package InfoCollection

import (
	"ColdVerify_server/lib"
	"ColdVerify_server/logs"
	"github.com/beego/beego/v2/adapter/orm"
	orm2 "github.com/beego/beego/v2/client/orm"
	_ "github.com/go-sql-driver/mysql"
	"log"
	"time"
)

// 模版
type InfoTemplateMap struct {
	Id int `orm:"column(ID);size(11);auto;pk"`

	T_id              string `orm:"index,size(256);null"` // 标题
	T_InfoTemplate_id string `orm:"size(256);null"`       // 标题
	T_label           int    `orm:"size(2);default(1)"`   // 数据类型

	//1：文本
	//2：数值
	//3：设备多选 （1|2|3|4|5|6|）
	//4：设备单选选 （1）
	//7：当期时间（2022年01月19日）
	//9：当期时间（开始与结束时间）（2022-01-01 13:08|2022-01-02 13:08）
	//10：图片
	//11：CAD
	T_name string `orm:"size(256);null"`     // 标签名称
	T_text string `orm:"size(256);null"`     // 描述
	T_sort int    `orm:"size(2);default(1)"` // 排序

	CreateTime time.Time `orm:"column(create_time);type(timestamp);null;auto_now_add"` //auto_now_add 第一次保存时才设置时间
	UpdateTime time.Time `orm:"column(update_time);type(timestamp);null;auto_now"`     //auto_now 每次 model 保存时都会对时间自动更新
}

type InfoTemplateMap_R struct {
	T_id    string
	T_label int    // 来源
	T_name  string // 标签名称
	T_text  string // 描述
	T_sort  int    // 排序
}

func (t *InfoTemplateMap) TableName() string {
	return "info_template_map" // 数据库名称   // ************** 替换 FormulaList **************
}

func init() {

	//注册模型
	orm.RegisterModel(new(InfoTemplateMap))

}

// -------------------------------------------------------------
func InfoTemplateMapToInfoTemplateMap_R(T InfoTemplateMap) (T_r InfoTemplateMap_R) {
	T_r.T_id = T.T_id
	T_r.T_label = T.T_label
	T_r.T_name = T.T_name
	T_r.T_text = T.T_text
	T_r.T_sort = T.T_sort
	return T_r
}

// ---------------- 特殊方法 -------------------

// 获取 ById
func Read_InfoTemplateMap_ById(id int) (r InfoTemplateMap, is bool) {
	o := orm.NewOrm()
	r = InfoTemplateMap{Id: id}
	err := o.Read(&r) // o.Read(&r,"Tokey") 如果不是 主键 就得指定字段名
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return r, false
	}
	return r, true
}

// 获取 By
func Read_InfoTemplateMap(T_id string) (r InfoTemplateMap, is bool) {

	o := orm.NewOrm()
	qs := o.QueryTable(new(InfoTemplateMap))
	err := qs.Filter("T_id", T_id).One(&r)
	if err != nil {
		return r, false
	}

	return r, true
}

// 添加
func Add_InfoTemplateMap(r InfoTemplateMap) (id int64, is bool) {
	o := orm.NewOrm()

	// 生成编号
	rand_x := 0
	for true {
		r.T_id = lib.GetRandstring(4, "", int64(rand_x)) // 1,336,336
		err := o.Read(&r, "T_id")                        // o.Read(&r,"Tokey") 如果不是 主键 就得指定字段名
		if err != nil {
			break
		}
		rand_x += 1
	}

	id, err := o.Insert(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return 0, false
	}

	return id, true
}

// 添加
func Add_InfoTemplateMapMulti(r []InfoTemplateMap) (id int64, is bool) {
	o := orm.NewOrm()
	n, err := o.InsertMulti(len(r), r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return n, false
	}

	return n, true
}

// 删除
func Delete_InfoTemplateMap(v InfoTemplateMap) bool {
	o := orm.NewOrm()
	if num, err := o.Delete(&v); err == nil {
		logs.Println("Number of records deleted in database:", num)
	} else {
		return false
	}

	return true
}

// 修改
func Update_InfoTemplateMap(m InfoTemplateMap, cols ...string) bool {
	o := orm.NewOrm()
	if num, err := o.Update(&m, cols...); err == nil {
		log.Println("Number of records updated in database:", num)
		return true
	}
	return false
}

// 获取列表
func Read_InfoTemplateMap_List(T_InfoTemplate_id string, T_sort, T_flow_sort int) ([]InfoTemplateMap_R, int64) {

	o := orm.NewOrm()
	// 也可以直接使用 Model 结构体作为表名
	var r []InfoTemplateMap
	qs := o.QueryTable(new(InfoTemplateMap))
	cond := orm.NewCondition()
	cond1 := cond.And("T_InfoTemplate_id", T_InfoTemplate_id) // .AndNot("status__in", 1).Or("profile__age__gt", 2000)
	// 排序 默认升序

	if T_sort == 2 {
		qs.SetCond((*orm2.Condition)(cond1)).OrderBy("-T_sort").All(&r)
		//} else if T_flow_sort == 1 {
		//	qs.SetCond((*orm2.Condition)(cond1)).OrderBy("T_flow_sort").All(&r)
		//} else if T_flow_sort == 2 {
		//	qs.SetCond((*orm2.Condition)(cond1)).OrderBy("-T_flow_sort").All(&r)
	} else {
		qs.SetCond((*orm2.Condition)(cond1)).OrderBy("T_sort").All(&r)
	}
	cnt, _ := qs.SetCond((*orm2.Condition)(cond1)).Count()

	// 转换
	var InfoTemplateMap_r []InfoTemplateMap_R
	for _, v := range r {
		InfoTemplateMap_r = append(InfoTemplateMap_r, InfoTemplateMapToInfoTemplateMap_R(v))
	}

	return InfoTemplateMap_r, cnt
}

func Read_InfoTemplateMap_List_For_Data(T_InfoTemplate_id string) []InfoTemplateMap {

	o := orm.NewOrm()
	// 也可以直接使用 Model 结构体作为表名
	var r []InfoTemplateMap
	qs := o.QueryTable(new(InfoTemplateMap))

	cond := orm.NewCondition()
	cond = cond.And("T_InfoTemplate_id", T_InfoTemplate_id)
	qs.SetCond((*orm2.Condition)(cond)).OrderBy("T_sort").All(&r)

	return r
}

func InfoTemplateMap_RToMap(T []InfoTemplateMap_R) map[string]string {
	maps := make(map[string]string, len(T))
	for _, v := range T {
		maps[v.T_id] = v.T_name
	}
	return maps
}
